########################################################################
# 
# Integrative multi-platform meta-analysis of gene expression profiles
# in pancreatic ductal adenocarcinoma patients for identifying novel
# diagnostic biomarkers
#
# Script written by: Francisco M. Ortuo Guzmn
# Email: fortuno@ugr.es
# Date: 02/05/2016
#
# Working for R version 3.1.2 (2014-10-31)
########################################################################

# Use previously published individual analysis for Illumina and Affymetrix:
illumina_data <- "IlluminaData.RData"
affymetrix_data <- "AffymetrixData.RData"
load(illumina_data)
load(affymetrix_data)

# Import require libraries and install them 
# in case they are not installed yet:

# 1) arrayQualityMetrics and dependencies
if (!("arrayQualityMetrics" %in% installed.packages())) { 
  source("http://www.bioconductor.org/biocLite.R"); 
  biocLite("Cairo")
  biocLite("Hmisc")
  biocLite("arrayQualityMetrics");
}
require(arrayQualityMetrics);

# 2) lumiHumanAll and other annotation packages
if (!("lumiHumanAll.db" %in% installed.packages())) { 
  source("http://www.bioconductor.org/biocLite.R"); 
  biocLite("lumiHumanAll.db");
}
require(lumiHumanAll.db);
if (!("oligoClasses" %in% installed.packages())) { 
  source("http://www.bioconductor.org/biocLite.R"); 
  biocLite("oligoClasses");
}
if (!("hugene10sttranscriptcluster.db" %in% installed.packages())) { 
  source("https://bioconductor.org/biocLite.R")
  biocLite("hugene10sttranscriptcluster.db")
}
require("hugene10sttranscriptcluster.db")

# 3) lumi
if (!("lumi" %in% installed.packages())) { 
  source("http://www.bioconductor.org/biocLite.R"); 
  biocLite("lumi");
}
require(lumi);

# 4) limma
if (!("limma" %in% installed.packages())) { 
  source("http://www.bioconductor.org/biocLite.R"); 
  biocLite("limma");
}
require(limma);

# 5) virtualArray and dependencies
if (!("tseries" %in% installed.packages())) {
  install.packages("tseries")
}
if (!("outliers" %in% installed.packages())) {
  install.packages("outliers")
}
if (!("virtualArray" %in% installed.packages())) { 
  # Depending on R version, you would need to install from source
  install.packages("virtualArray") 
}
require(virtualArray);

# 6) ROCR
if (!("ROCR" %in% installed.packages())) { 
  # Depending on R version, you would need to install from source
  install.packages("ROCR") 
}
require(ROCR);

# 6) topGO
if (!("topGO" %in% installed.packages())) { 
  source("http://www.bioconductor.org/biocLite.R"); 
  biocLite("topGO");
}
require(topGO);

# Gene annotation for individual studies: ID, SYMBOL, Entrez ID, Gene Name.
if (require(lumiHumanAll.db) & require(annotate)) {
  genesID<-rownames(reduced.lumi)
  lumiGeneSymbol <- getSYMBOL(genesID, 'lumiHumanAll') # SYMBOL
  lumiGeneEG<-getEG(genesID,'lumiHumanAll') # Entrez Gene ID
  lumiGeneName <- sapply(lookUp(genesID, 'lumiHumanAll', 'GENENAME'), function(x) x[1]) # Gene Name  
  
}
annotation(reduced.lumi)<-"lumiHumanAll"

affyGeneSymbol <- unlist(mget(rownames(OligoEset),hugene10sttranscriptclusterSYMBOL))
affyGeneEG <- unlist(mget(rownames(OligoEset),hugene10sttranscriptclusterENTREZID)) # The interesting genes: Pos vs Control All
affyGeneName <- unlist(mget(rownames(OligoEset),hugene10sttranscriptclusterGENENAME))

# Get number of samples per technology
numAffy = dim(OligoEset)[2];
numLumi = dim(reduced.lumi)[2];

#####################################################
# STEP 1: DATA TRANSFORMATION BASED ON BOXPLOTS
#####################################################

# A. Boxplot comparison
numAffyPre = sum(targetsAffy$Target=="Pre")
numAffyControl = sum(targetsAffy$Target=="Control")
numLumiPre = sum(targetsLumi$Target=="Paciente")
total = numAffyPre + numAffyControl + numLumiPre
boxplot(exprs(OligoEset)[,targetsAffy$Target=="Pre"], at=1:numAffyPre, xlim=c(0.5,(total+0.5)),add=F, las=3, col="cadetblue3", outline=FALSE, xaxt='n', whisklty = 1, ylab=('log2(exp)'))
boxplot(exprs(OligoEset)[,targetsAffy$Target=="Control"], at=(numAffyPre+1):(numAffyPre+numAffyControl), xlim=c(0.5,(total+0.5)),add=T, las=3, col="firebrick1", outline=FALSE, xaxt='n', whisklty = 1)
boxplot(exprs(reduced.lumi)[,targetsLumi$Target=="Paciente"], at=(numAffyPre+numAffyControl+1):total, xlim=c(0.5,(total+0.5)), add=T,las=3, col="darkseagreen3", outline=FALSE, xaxt='n',  whisklty = 1)
adjustcolor("grey",alpha.f = 0.3)
grid(nx=NA, ny=NULL) 
title('(A) Illumina and Affymetrix dataset before ComBat batch removal', cex.main=0.8)
legend(58,5,c('Cohort 1 (18)', 'Controls (18)', 'Cohort 2 (36)'), inset=.02, fill=c('cadetblue3','firebrick1', 'darkseagreen3'), cex=0.7)

# B. Logaritmic transformation: 
# Both datasets were already transformed - Transformation not needed.

# C. Bits coverage transfomration:
# Both technologies used same number of bits - Transformation not needed.



#####################################################
# STEP 2: INTEGRATION USING VIRTUAL ARRAY LIBRARY
#####################################################

# Virtual array automatically integrates platforms using
# the following rules:
# 1. Gene symbol integration Integracin por smbolos.
# 2. Integration of repeated genes by expression median.
# 3. Batch effect removal.
virtualArray <- virtualArrayExpressionSets(all_expression_sets=c("OligoEset", "reduced.lumi"))


############################################################
# STEP 3: QUALITY CONTROL ANALYSIS OF INTEGRATED DATA
############################################################
arrayQualityMetrics(expressionset = virtualArray, outdir = "./integrado/Quality_Report_virtualArray" , force = TRUE, do.logtransform = TRUE);
virtualTargets<- rbind(targetsAffy, targetsLumi)
controlsExp <- exprs(virtualArray)[,targetsAffy$Target=="Control"]
boxplot(exprs(virtualArray)[,virtualTargets$Target=="Pre"], at=1:numAffyPre, xlim=c(0.5,(total+0.5)),add=F, las=3, col="cadetblue3", outline=FALSE, xaxt='n', whisklty = 1, ylab=('log2(exp)'))
boxplot(controlsExp[,1:numAffyControl], at=(numAffyPre+1):(numAffyPre+numAffyControl), xlim=c(0.5,(total+0.5)),add=T, las=3, col="firebrick1", outline=FALSE, xaxt='n', whisklty = 1)
boxplot(exprs(virtualArray)[,virtualTargets$Target=="Paciente"], at=(numAffyPre+numAffyControl+1):total, xlim=c(0.5,(total+0.5)), add=T,las=3, col="darkseagreen3", outline=FALSE, xaxt='n',  whisklty = 1)
adjustcolor("grey",alpha.f = 0.3)
grid(nx=NA, ny=NULL) 
title('(B) Illumina and Affymetrix dataset after ComBat batch removal', cex.main=0.8)
legend(58,6,c('Cohort 1 (18)', 'Controls (18)', 'Cohort 2 (36)'), inset=.02, fill=c('cadetblue3','firebrick1', 'darkseagreen3'), cex=0.7)

############################################################
# STEP 4: ANALYSIS OF DIFFERENTIALLY EXPRESSED GENES
############################################################

# Integrate comparison groups to make contrast study
virtualTargets<- rbind(targetsAffy, targetsLumi)
virtualTargets$Target[virtualTargets$Target=="Paciente"]<-"Pre"

# Run lmFit for differential expression study
niveles <- c("Control", "Pre", "Pos") 
group <- factor(virtualTargets$Target, levels=niveles)
design <- model.matrix(~0+group)
fit <- lmFit(virtualArray, design)
cont.matrix <- makeContrasts(groupPre-groupControl,groupPos-groupControl,groupPos-groupPre, levels=design)
fit2 <- contrasts.fit(fit, cont.matrix)
fit2 <- eBayes(fit2)

############################################################
# STEP 5: FILTER SIGNIFICANT DIFFERENTIALLY EXPRESSED GENES
############################################################

# Comparison parameters:
#  * pvalue: 0.01
#  * adjust: false discovery rate (FDR) (Benjamini & Hochberg)
#  * foldchange: 1.5
p_threshold=0.01
p_adj_method="BH"
lfc_adj = log2(1.5);
tableExp<-topTable(fit2, coef=1,adjust=p_adj_method, lfc=lfc_adj, p.value=p_threshold, sort.by="logFC",number=nrow(fit2))

# Add annotation columns to differentially expressed genes
SYMBOLS <- rownames(tableExp)
ENTREZ <- lumiGeneEG[match(SYMBOLS,lumiGeneSymbol)]
GENENAME <- lumiGeneName[match(SYMBOLS,lumiGeneSymbol)]
tableExp<-cbind(ENTREZ, SYMBOLS, GENENAME, tableExp[,1:ncol(tableExp)])

# Save results in a tab-separated file
resultsFile<-"Integrated-PrevsControl-results.txt"
write.table(tableExp,resultsFile,sep="\t",dec=".",append=FALSE,row.names=FALSE)

##########################################################################
# STEP 6: COMPARISON BETWEEN INTEGRATION AND INDIVIDUAL ANALYSIS
##########################################################################

# Get gene symbols from genes differentially expressed in each study
affySymbols <- affyTable$SYMBOL[!is.na(affyTable$SYMBOL)]
lumiSymbols <- tableLumi$geneSymbol[!is.na(tableLumi$geneSymbol)]
virtualSymbols <-  tableExp$SYMBOL

# Join all unique gene symbols
allExpressedGenes <- unique(union(affySymbols, union(lumiSymbols,virtualSymbols)))

# Get a Venn diagram with intersections of differentially expressed genes
Affyinc <- is.element(allExpressedGenes, affySymbols)
Lumiinc <- is.element(allExpressedGenes, lumiSymbols)
Virtualinc <- is.element(allExpressedGenes, virtualSymbols)
coincidences<-vennCounts(cbind(Affyinc,Lumiinc,Virtualinc))
vennDiagram(coincidences, circle.col=c("red","green","blue"), lwd=1, cex=1, names=c("Affymetrix analysis","Illumina analysis","Integrated meta-analysis"))

# Get coincidences (intersection) genes between studies.
coincidenceAffyLumi <- intersect(affySymbols, lumiSymbols)
coincidenceAffyInt <- intersect(affySymbols, virtualSymbols)
coincidenceLumiInt <- intersect(lumiSymbols, virtualSymbols)
allcoincidences <- intersect(coincidenceAffyLumi, coincidenceAffyInt)

only_integrated <- !(virtualSymbols %in% coincidenceAffyInt) & !(virtualSymbols %in% coincidenceLumiInt)
integrated <- virtualSymbols[only_integrated]


##########################################################################
# STEP 7: SENSITIVITY/SPECIFICITY ANALYSIS FOR COMMON GENES
##########################################################################

# Get expression and targets from 5 common genes
lumiGenes <- lumiGeneSymbol[lumiGeneSymbol %in% allcoincidences]
lumiExprs <- exprs(reduced.lumi[names(lumiGenes)])
lumiTarget <- targetsLumi$Target

affyGenes <- affyGeneSymbol[affyGeneSymbol %in% allcoincidences]
affySamples <- targetsAffy[targetsAffy$Target %in% c("Pre", "Control"),]$Name
affyExprs <- exprs(OligoEset[names(affyGenes)])[,affySamples]
affyTarget <- targetsAffy[targetsAffy$Target %in% c("Pre", "Control"),]$Target

intSamples <- virtualTargets[virtualTargets$Target %in% c("Pre", "Control"),]$Name
intExprs <- exprs(virtualArray[allcoincidences])[,intSamples]
intTarget <- virtualTargets[virtualTargets$Target %in% c("Pre", "Control"),]$Target

# Calculate ROC curve for each gene in the three analyses
for(gene in allcoincidences) {

  lumiContrast <- lumiTarget == "Control"  
  affyContrast <- affyTarget == "Control"  
  intContrast <- intTarget == "Control"
  
  fc <- mean(lumiExprs[lumiGenes %in% gene,lumiContrast])/mean(lumiExprs[lumiGenes %in% gene,!lumiContrast])   
  if (fc < 1){
    lumiContrast <- lumiTarget != "Control"  
    affyContrast <- affyTarget != "Control"   
    intContrast <- intTarget != "Control"
  }
  
  predlumi <- prediction(lumiExprs[lumiGenes %in% gene,], lumiContrast)
  perflumi <- performance(predlumi,"auc")
  auclumi <- perflumi@y.values[[1]]
  roclumi<-performance(predlumi,"tpr","fpr")

  predaffy <- prediction(affyExprs[affyGenes %in% gene,], affyContrast)
  perfaffy <- performance(predaffy,"auc")
  aucaffy <- perfaffy@y.values[[1]]
  rocaffy<-performance(predaffy,"tpr","fpr")

  predint <- prediction(intExprs[rownames(intExprs) %in% gene,], intContrast)
  perfint <- performance(predint,"auc")
  aucint <- perfint@y.values[[1]]
  rocint <-performance(predint,"tpr","fpr")
  
  png(paste(sep="", gene, "_ROC.png"))
  title<-paste(sep="", gene, " ROC Curve")
  plot(roclumi, main=title, ylab="Sensitivity", xlab="1-Specificity", col="green", cex.lab=1.5, cex.axis=1.5, cex.main=1.5, lwd=2)
  plot(rocaffy, add = TRUE, col="red", cex=2, lwd=2)
  plot(rocint, add = TRUE, col="blue", cex=2, lwd=2)  
  text(x=0.4, y=0.8, labels = paste(sep="", "AUC = ", round(auclumi,digits=2)), col ="green", cex=1.5) 
  text(x=0.45, y=0.7, labels = paste(sep="", "AUC = ", round(aucaffy,digits=2)), col ="red", cex=1.5) 
  text(x=0.5, y=0.6, labels = paste(sep="", "AUC = ", round(aucint,digits=2)), col ="blue", cex=1.5) 
  legend("bottomright", c("Illumina", "Affymetrix", "Integrated"), col=c("green", "red", "blue"), lwd=2, cex=1.5)
  dev.off()
}

############################################################
# STEP 8: GO GENE ENRICHMENT ANALYSIS
############################################################

# Select gained genes 
arrayList <- fit2$p.value[,1]
selectedGenes <- arrayList[as.character(integrated)]

# BP ONTOLOGY
sampleGOdata <- new("topGOdata", ontology = "BP", allGenes = selectedGenes, geneSelec = topDiffGenes, nodeSize = 5, annotationFun = annFUN.org, mapping = "hugene10sttranscriptcluster.db", ID = "symbol")
resultFisher <- runTest(sampleGOdata, algorithm = "classic", statistic = "fisher")
resultKS <- runTest(sampleGOdata, algorithm = "classic", statistic = "ks")
resultKS.elim <- runTest(sampleGOdata, algorithm = "elim", statistic = "ks")
allRes <- GenTable(sampleGOdata, classicFisher = resultFisher, classicKS = resultKS, elimKS = resultKS.elim, orderBy = "elimKS", ranksOf = "classicFisher", topNodes = 10)
write.table(allRes, "BP_GO_enrichment_analysis.tsv", sep="\t")

# CC ONTOLOGY
sampleGOdata <- new("topGOdata", ontology = "CC", allGenes = selectedGenes, geneSelec = topDiffGenes, nodeSize = 5, annotationFun = annFUN.org, mapping = "hugene10sttranscriptcluster.db", ID = "symbol")
resultFisher <- runTest(sampleGOdata, algorithm = "classic", statistic = "fisher")
resultKS <- runTest(sampleGOdata, algorithm = "classic", statistic = "ks")
resultKS.elim <- runTest(sampleGOdata, algorithm = "elim", statistic = "ks")
allRes <- GenTable(sampleGOdata, classicFisher = resultFisher, classicKS = resultKS, elimKS = resultKS.elim, orderBy = "elimKS", ranksOf = "classicFisher", topNodes = 10)
write.table(allRes, "CC_GO_enrichment_analysis.tsv", sep="\t")

# MF ONTOLOGY
sampleGOdata <- new("topGOdata", ontology = "MF", allGenes = selectedGenes, geneSelec = topDiffGenes, nodeSize = 5, annotationFun = annFUN.org, mapping = "hugene10sttranscriptcluster.db", ID = "symbol")
resultFisher <- runTest(sampleGOdata, algorithm = "classic", statistic = "fisher")
resultKS <- runTest(sampleGOdata, algorithm = "classic", statistic = "ks")
resultKS.elim <- runTest(sampleGOdata, algorithm = "elim", statistic = "ks")
allRes <- GenTable(sampleGOdata, classicFisher = resultFisher, classicKS = resultKS, elimKS = resultKS.elim, orderBy = "elimKS", ranksOf = "classicFisher", topNodes = 10)
write.table(allRes, "MF_GO_enrichment_analysis.tsv", sep="\t")